
#ifndef ARDUIONO_ICOMMESSAGE
#define ARDUIONO_ICOMMESSAGE

#include <Arduino.h>

// What is the max lengh of an ICOM Message  Should be 14 but 16 gives some protection
static const uint8_t icomMessageMaxLength = 32;
/// What is the value of the first two bytes of an Icom Message (only one stored in this class)
static const uint8_t icomMessageStartByte = 0xFE;
///  What is the value of the last byte of an Icom Message
static const uint8_t icomMessageStopByte = 0xFD;

/// determines if an ID is a controller or not...
inline bool isControllerID(const uint8_t x) { return ((x & 0xf0) == 0xe0); }
/**
* @author James Michener K9JM  (c) 2012
*
* Routine that is a container class and interprets / makes
* IcomMessages
* Only the first of the two icomMessageStartBytes are stored
* in this message
*/


class IcomMessage
{
public:
	IcomMessage(void);
	~IcomMessage(void) {};
	/// add byte to buffer  return true if ok
	bool add(const uint8_t x);
	inline uint8_t * get() { return buffer; }
	inline uint8_t getLen() { return m_Counter; }
	inline void setInUse()  { m_Counter = 0xff; }
	inline bool isMarkedInUse() { return m_Counter == 0xff; }
	inline bool isEmpty() { return !m_Counter; }
	inline void setMTY() { m_Counter = 0; }
	inline uint8_t getSourceID() const { return buffer[2]; }
	inline uint8_t getDestinationID() const { return buffer[1]; }
	inline uint8_t getCommand() const { return buffer[3]; }

	inline bool isMode26Command() const { return buffer[3] == 0x26; }
	inline bool isModeCommand() const { return (buffer[3] == 4 || buffer[3] == 1 || buffer[3] == 6);  }
//	inline bool isModeBW() const { return  m_Counter == 7  && (buffer[3] == 4 || buffer[3] == 1  || buffer[3] == 6) ; }
//	inline bool isModeOnly() const { return  m_Counter == 6  && (buffer[3] == 4 || buffer[3] == 1  || buffer[3] == 6) ; }
	inline uint8_t getMode()  const { return buffer[4]; }
	inline uint8_t getBW()    const { return buffer[5]; }

	inline uint8_t getMode26()  const { return buffer[5]; }
	inline uint8_t getDataMode26() const { return buffer[6]; }
	inline uint8_t getBW26()    const { return buffer[7]; }

	// tests to see if special okay or 'no good' message
	inline bool isOkay()   const { return (m_Counter == 5 && buffer[3] == 0xfb); }
	inline bool isNG()     const { return (m_Counter == 5 && buffer[3] == 0xfa); }

	inline bool isFreqQuery() const { return  (m_Counter == 5 && buffer[3] == 3) ; }
	inline bool isFreq25Query() const { return  (m_Counter == 6 && buffer[3] == 0x25); }
	inline bool isModeBWQuery()  const { return  m_Counter == 5  && buffer[3] == 4 ; }
	inline bool isModeBW26Query()  const { return  m_Counter == 6 && buffer[3] == 0x26; }
	inline bool isVolumeQuery()  const { return  m_Counter == 6 && buffer[3] == 0x14 && buffer[4] == 1; }
	inline bool isSpecialQuery() const { return  m_Counter == 6 && buffer[3] == 0x1a && buffer[4] == 6; }

	inline bool isChangeFreqOrModeCommand() const { return (buffer[3] == 7 || buffer[3] == 8); }
	inline bool isChangeFreqCommand() const { return (buffer[3] == 5); }  // set freq only
	inline bool isSendOrSetFreqCommand() const { return (buffer[3] == 5 || !buffer[3]); }  // set freq only

	inline bool isChangeModeCommand() const { return (buffer[3] == 6); }

	inline bool isVFOCommand() const { return (buffer[3] == 7); }
	inline bool isVFOSubCommand() const  { return ( buffer[4] == 0xd1); } 
	inline bool isVFOMainCommand() const { return ( buffer[4] == 0xd0); } 

	void extendlongby(const uint8_t x);

	// does this command contain frequency data
	bool isFreq() const;
	bool isFreq25() const;  // for new command 25 frequency
	// only call after establishing the command contains freq data
	unsigned long getFreq() const;
	unsigned long getFreq25() const;  // for new command 25
	/// if we know it is a Frequency 25 command,  this returns the VFO
	inline uint8_t getFreq25VFO() const { return buffer[4]; }
	inline uint8_t getMode26VFO() const { return buffer[4]; }

	/// return false if there is something obviously wrong with it
	/// does sanity checks.  Wish here was a check sum or CRC!!
	bool ValidateReceiveMessage();

	bool isEqual(const IcomMessage & x);

	// Routines for building an icomMessage

	inline void swapSourceDestinationID() { uint8_t t = buffer[1]; buffer[1] = buffer[2]; buffer[2] = t; }
	void setSourceDestinationID(const uint8_t source, const uint8_t dest);
	inline void setDestinationID(const uint8_t dest) { buffer[1] = dest; }
	inline void setSourceID(const uint8_t source) { buffer[2] = source; }
	void setModeBW(const uint8_t mode, const uint8_t bw);
	void setFreq(const unsigned long freq);
	void setFreq25(const unsigned long freq, const uint8_t vfo);
	void setPower(const uint8_t value);
	void setSplitOff();  // make message into a split off message
	void setMainReadout(); 
    //builds a power message to query power
    void setPowerQuery();
    bool isPower();
	uint8_t getByteResponse(const uint8_t offset = 5);
	void setFreqCmd0(const unsigned long freq);
	void setModeBWCmd1(const uint8_t mode, const uint8_t bw);

	void copy(const IcomMessage & a);

	void dump() const;
    void dumpSpecial() const;

	// DATA

	// countes the bytes as they come in
	// otherwise is the byte count in message
	// if zero them mty,  if 0xff then in use but no data.
	uint8_t m_Counter;

	// The actual message
	uint8_t buffer[icomMessageMaxLength];

private:

	// used by getFreq()
	inline uint8_t bcdByte(const uint8_t x) const { return  (((x & 0xf0) >> 4) * 10) + (x & 0x0f); }
	// used by setFreq()
	// input is between 0 and 99 decimal.  output 0 to 0x99
	inline uint8_t bcdByteEncode(const uint8_t x) const { return ((x / 10) << 4) + (x % 10); }


};

#endif // ARDUIONO_ICOMMESSAGE
